/***************************************************************************
                          krpmbuilder.h  -  description
                             -------------------
    begin                : Don Aug  1 15:56:02 CEST 2002
    copyright            : (C) 2002 by Olaf Lueg
                           (C) 2005 by Wolfgang Beutner
    email                : olueg@olsd.de
                           wolfgang.beutner(at)gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KRPMBUILDER_H
#define KRPMBUILDER_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kapp.h>
#include <qwidget.h>
#include <qlineedit.h>
#include <klistbox.h>
#include <kwizard.h>
#include <kprocess.h>
#include <kpushbutton.h>
#include <kio/job.h>


class ProjectPage;
class FilesPage;
class PackagePage;
class StartPage;
class FinishPage;
class BuildPage;
class KComboBox;
class KAboutApplication;

/** \class KRPMBuilder
  * \brief This is the base class of the project KRPMBuilder
  * \author Wolfgang Beutner <wolfgang.beutner(at)gmx.de
  * \version 1.1.1
  * \date 04.03.2005
  */
class KRPMBuilder : public KWizard
{
  Q_OBJECT 
  public:
    /** construtor */
    KRPMBuilder(QWidget* parent=0, const char *name=0);
    /** destructor */
    ~KRPMBuilder();
  /** No descriptions */
  void readConfig();
  /** No descriptions */
  void saveConfig();
  /** No descriptions */
  void runRPM(bool buildBinary,bool buildSource,QString prozessor, QString filename);
  /** No descriptions */
  bool hasEmptyTags();
private slots: // Private slots
  /** No descriptions */
  void slotProjectOpen();
  /** No descriptions */
  void slotCreateRPM();
  /** No descriptions */
  void slotNew();
  /** No descriptions */
  void slotSave();
public slots: // Public slots
  /** No descriptions */
  void slotAbout();
  /** No descriptions */
  void slotQuit();
  /** No description */
  void slotSourceSearch();
  /** No descriptions */
  void slotUpdateInfo(KProcess*,char *buf , int len);
  /** No descriptions */
  void slotRpmDone(KProcess*);
  /** No descriptions */
  void slotPatchHighlighted(int index);
  
  /** Gets called when an entry in the fileslist is selected
    * \param index Index of the selected entry.
    */
  
  void slotFileHighlighted(int index);
  
  /** No descriptions */
  void slotFileAdd();
  /** No descriptions */
  void slotFileDelete();
  /** No descriptions */
  void slotFileMoveDown();
  /** No descriptions */
  void slotFileMoveUp();
  /** No descriptions */
  void slotPatchMoveDown();
  /** No descriptions */
  void slotPatchMoveUp();
  /** No descriptions */
  void slotPatchDelete();
  /** No descriptions */
  void slotPatchAdd();
  /** No descriptions */
  void slotPatchSearch();
  /** No descriptions */
  void slotFileSearch();
  /** No descriptions */
  void slotCopyJobResult(KIO::Job *job);
  /** No descriptions */
  void slotSettings();
  /** Gets called when the stop-button was pressed during rpm-build*/
  void slotStopRPM();
  /**
    * Gets called when the job copying files on project reload finishes.
    * \param Pointer to the job that finished.
    */
    
  
  void slotStartupCopyResult( KIO::Job *job );
  
  /**
    * Gets called when the job deleting a file from the rpm SOURCES
    * directory finished.
    * \param Pointer to the job that finished
    */
    
   void slotDeleteResult( KIO::Job *job );
   
private: // Private methods
  /** No descriptions */
  bool findItem(KComboBox *combo,const QString &item);

  /**
    * Check if any of the possible Fields has changed its content.
    * \return true if at least one field was modified
    */
    
  bool projectIsModified();
  
  /**
    * Control the enabled status of the moveup movedown buttons.
    * \param index  number of the entry to behighlighted
    * \itemCount number of items in listbox
    * \upButton button for moving items up
    * \downButton button for moving item down
    */
    
  void enableUpDownButtons( const int index, const int itemCount, KPushButton *upButton, KPushButton *downButton );
  
  /**
    * Move a selected entry up one position in one of the
    * listboxes.
    * \param  Listbox  the listbox where the move was triggered
    */
   
  void moveItemDown( KListBox *Listbox );
  
  
  /**
    * Move a selected entry up one position in one of the
    * listboxes.
    * \param  Listbox  the listbox where the move was triggered
    */
   
  void moveItemUp( KListBox *Listbox );
  
  /**
    * Copy the selected file to the rpm SOURCE directory.
    * It is checked, that the file is not selected from
    * that directory itself, bacuse that would truncate
    * the file to zeo bytes.
    * \param fromURL the Url of the file to be copied
    */
    
  void copyFileToSOURCE( KURL & fromURL );
  
  /**
    * Handle the dialogue for file selection after either
    * of the two search buttons in the filesPage has been
    * pressed. The newly selected file must not match one
    * of the following conditions:
    *    selected as the primary source for the build
    *    is entered in the editbox of the other tab
    *    is already shown in one of the lists.
    * \param ListBox listbox that started the dialogue
    * \param Filetypes a list of filenames allowed for the selection
    * \return KURL of selected file or empty KURL
    */
    
  KURL handleFileSearch( KListBox *ListBox, QString filetypes );
  
  /**
    * We check that the selected files are accessible and copy them
    * to the target directory (rpm/SOURCES). If on of the desired files
    * is not in place we are in trouble.
    */
    
  void copyAllFilesToSOURCE();
  
  /**
    * Deletes the specified file from the rpm SOURCES directory
    * \param filename  Name of file to be deleted
    */
  
  void deleteFileFromSOURCE( QString &filename );
    
  //! Shows the About-Dialog
  KAboutApplication *about;
  //! Logo of KRPMBuilder
  QPixmap pLogo;
  //! Spacer to fill the gap between logo and right border
  QPixmap pSpacer;
  //! Startup dialog
  StartPage *startPage;
  //! Page to enter information about the project
  ProjectPage *projectPage;
  //! Page to select additional sources and patches
  FilesPage *filesPage;
  //! Information about the package itself
  PackagePage *packagePage;
  //! Additional information for the build process
  FinishPage *finishPage;
  //! The build process is controlled from here
  BuildPage *buildPage;
  //! True if we build the binary package
  bool binaryRPM;
  //! True if a source package is built
  bool sourceRPM;
  //! To control a process that is started from within the application
  KShellProcess *proc;
 //! Path to SOURCES directory for build process  KURL RPMsourceURL;
  KURL RPMsourceURL;
  //! Path to SPECS directory for build process
  KURL RPMspecURL;
  //! URL of selected patch before it is added to list
  KURL patchURL;
  //! URL of additional source before it is added to list
  KURL sourceURL;
  //! Path of the file selected from projectpage as primary source
  KURL sourceDir;
  //! True if an error occured during filecopy.
  bool copyError;
  //! True if paths for the SPEC and SOURCES drirectory are set
  bool hasPaths;
  //! Set to true if the project needs to be saved
  bool saveProject;
  /** Pointer to the widget, where the name of
    *  successfully copied file is moved to. 
    */
  QWidget *targetWidget;
};

#endif
